/*
 * Copyright (C) 2008-2009  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static int
tcp_connect (struct addrinfo *ai)
{
  int fd;
  fd = socket (ai->ai_family, SOCK_STREAM, 0);
  if (fd < 0)
    return -1;
  if (connect (fd, ai->ai_addr, ai->ai_addrlen) < 0)
    {
      close (fd);
      return -1;
    }
  return fd;
}

static int
tcp_connect_list (struct addrinfo *ai)
{
  int fd = -1;
  for (; ai; ai = ai->ai_next)
    {
      fd = tcp_connect (ai);
      if (fd >= 0)
        {
          return fd;
        }
    }
  return fd;
}

int
hc_tcp_connect (char *server, char *service)
{
  struct addrinfo *ai = NULL;
  int fd;
  if (getaddrinfo (server, service, NULL, &ai) < 0)
    return -1;
  fd = tcp_connect_list (ai);
  freeaddrinfo (ai);
  return fd;
}

#ifdef TEST
int
main (int argc, char **argv)
{
  char *server;
  char *service;
  int fd;
  server = (argc >= 2) ? argv[1] : "localhost";
  service = (argc >= 3) ? argv[2] : "110";
  fd = hc_tcp_connect (server, service);
  if (fd > 0)
    close (fd);
  return 0;
}
#endif
